'use strict';

var _powerAssertRecorder = function () { function PowerAssertRecorder() { this.captured = []; } PowerAssertRecorder.prototype._capt = function _capt(value, espath) { this.captured.push({ value: value, espath: espath }); return value; }; PowerAssertRecorder.prototype._expr = function _expr(value, source) { return { powerAssertContext: { value: value, events: this.captured }, source: source }; }; return PowerAssertRecorder; }();
/* eslint-env mocha */

var _powerAssert = require('power-assert');

var _powerAssert2 = _interopRequireDefault(_powerAssert);

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _sinon = require('sinon');

var _index = require('../index');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('FetcherTest', function () {
    describe('base', function () {
        it('should create RequestOptions and fullUrl', function () {
            var _rec = new _powerAssertRecorder(),
                _rec2 = new _powerAssertRecorder(),
                _rec3 = new _powerAssertRecorder();

            var options = {
                baseUrl: '/api',
                headers: {
                    'Accept-Language': 'ru'
                },
                method: 'GET'
            };
            var baseOptions = new _index.Fetcher(options);

            var result = {
                body: null,
                headers: {
                    'Accept-Language': 'ru'
                },
                method: 'GET'
            };
            (0, _powerAssert2.default)(_rec._expr(_rec._capt(_rec._capt(_rec._capt(baseOptions, 'arguments/0/left/object').fullUrl, 'arguments/0/left') === _rec._capt(_rec._capt(options, 'arguments/0/right/object').baseUrl, 'arguments/0/right'), 'arguments/0'), {
                content: 'assert(baseOptions.fullUrl === options.baseUrl)',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 33
            }));
            _powerAssert2.default.deepEqual(_rec2._expr(_rec2._capt(_rec2._capt(baseOptions, 'arguments/0/object').options, 'arguments/0'), {
                content: 'assert.deepEqual(baseOptions.options, result)',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 34
            }), _rec3._expr(_rec3._capt(result, 'arguments/1'), {
                content: 'assert.deepEqual(baseOptions.options, result)',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 34
            }));
        });

        it('should merge parameters in new instance', function () {
            var _rec4 = new _powerAssertRecorder(),
                _rec5 = new _powerAssertRecorder();

            var aOptions = {
                method: 'GET'
            };
            var bOptions = {
                method: 'POST'
            };
            var a = new _index.Fetcher(aOptions);
            var b = a.copy(bOptions);
            (0, _powerAssert2.default)(_rec4._expr(_rec4._capt(_rec4._capt(a, 'arguments/0/left') !== _rec4._capt(b, 'arguments/0/right'), 'arguments/0'), {
                content: 'assert(a !== b)',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 46
            }));
            (0, _powerAssert2.default)(_rec5._expr(_rec5._capt(_rec5._capt(_rec5._capt(_rec5._capt(b, 'arguments/0/left/object/object').options, 'arguments/0/left/object').method, 'arguments/0/left') === 'POST', 'arguments/0'), {
                content: 'assert(b.options.method === \'POST\')',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 47
            }));
        });

        it('should merge headers', function () {
            var _rec6 = new _powerAssertRecorder(),
                _rec7 = new _powerAssertRecorder();

            var aOptions = {
                headers: {
                    a: 'a1',
                    c: 'c1'
                }
            };
            var bOptions = {
                headers: {
                    a: 'a2',
                    b: 'b1'
                }
            };
            var a = new _index.Fetcher(aOptions);
            var b = a.copy(bOptions);

            _powerAssert2.default.deepEqual(_rec6._expr(_rec6._capt(_rec6._capt(_rec6._capt(b, 'arguments/0/object/object').options, 'arguments/0/object').headers, 'arguments/0'), {
                content: 'assert.deepEqual(b.options.headers, { a: \'a2\', b: \'b1\', c: \'c1\' })',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 66
            }), _rec7._expr(_rec7._capt({
                a: 'a2',
                b: 'b1',
                c: 'c1'
            }, 'arguments/1'), {
                content: 'assert.deepEqual(b.options.headers, { a: \'a2\', b: \'b1\', c: \'c1\' })',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 66
            }));
        });

        it('should compose postProcess', function () {
            var _rec8 = new _powerAssertRecorder(),
                _rec9 = new _powerAssertRecorder(),
                _rec10 = new _powerAssertRecorder();

            var p1 = (0, _sinon.spy)(function (v) {
                return v + 1;
            });
            var aOptions = {
                postProcess: p1
            };
            var p2 = (0, _sinon.spy)(function (v) {
                return v + 2;
            });
            var bOptions = {
                postProcess: p2
            };
            var a = new _index.Fetcher(aOptions);
            var b = a.copy(bOptions);
            var result = b.postProcess(1);

            (0, _powerAssert2.default)(_rec8._expr(_rec8._capt(_rec8._capt(result, 'arguments/0/left') === 4, 'arguments/0'), {
                content: 'assert(result === 4)',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 86
            }));
            (0, _powerAssert2.default)(_rec9._expr(_rec9._capt(_rec9._capt(p1, 'arguments/0/callee/object').calledWith(1), 'arguments/0'), {
                content: 'assert(p1.calledWith(1))',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 87
            }));
            (0, _powerAssert2.default)(_rec10._expr(_rec10._capt(_rec10._capt(p2, 'arguments/0/callee/object').calledWith(2), 'arguments/0'), {
                content: 'assert(p2.calledWith(2))',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 88
            }));
        });

        it('should build fullUrl', function () {
            var _rec11 = new _powerAssertRecorder();

            var aOptions = {
                baseUrl: '/api'
            };
            var bOptions = {
                url: '/user'
            };
            var a = new _index.Fetcher(aOptions);
            var b = a.copy(bOptions);
            (0, _powerAssert2.default)(_rec11._expr(_rec11._capt(_rec11._capt(_rec11._capt(b, 'arguments/0/left/object').fullUrl, 'arguments/0/left') === '/api/user', 'arguments/0'), {
                content: 'assert(b.fullUrl === \'/api/user\')',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 100
            }));
        });
    });

    describe('serializeParams related', function () {
        it('should throw error if no createSerializeParams provided and params is set', function () {
            _powerAssert2.default.throws(function () {
                return new _index.Fetcher({
                    params: {
                        a: '1'
                    }
                });
            });
        });

        it('should build fullUrl with template', function () {
            var _rec12 = new _powerAssertRecorder();

            var aOptions = {
                baseUrl: '/api',
                serializeParams: (0, _index.createSerializeParams)(_querystring2.default.stringify)
            };
            var bOptions = {
                url: '/user/:id/:id2',
                params: {
                    id: '1',
                    id2: '2'
                }
            };
            var a = new _index.Fetcher(aOptions);
            var b = a.copy(bOptions);
            (0, _powerAssert2.default)(_rec12._expr(_rec12._capt(_rec12._capt(_rec12._capt(b, 'arguments/0/left/object').fullUrl, 'arguments/0/left') === '/api/user/1/2', 'arguments/0'), {
                content: 'assert(b.fullUrl === \'/api/user/1/2\')',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 129
            }));
        });

        it('should build fullUrl with query', function () {
            var _rec13 = new _powerAssertRecorder();

            var aOptions = {
                baseUrl: '/api',
                serializeParams: (0, _index.createSerializeParams)(_querystring2.default.stringify)
            };
            var bOptions = {
                url: '/user',
                params: {
                    id: '1',
                    id2: '2'
                }
            };
            var a = new _index.Fetcher(aOptions);
            var b = a.copy(bOptions);
            (0, _powerAssert2.default)(_rec13._expr(_rec13._capt(_rec13._capt(_rec13._capt(b, 'arguments/0/left/object').fullUrl, 'arguments/0/left') === '/api/user?id=1&id2=2', 'arguments/0'), {
                content: 'assert(b.fullUrl === \'/api/user?id=1&id2=2\')',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 146
            }));
        });

        it('should build fullUrl with custom placeholder RegExp', function () {
            var _rec14 = new _powerAssertRecorder();

            var aOptions = {
                baseUrl: '/api',
                serializeParams: (0, _index.createSerializeParams)(_querystring2.default.stringify, new RegExp('<([\\w]+)>', 'g'))
            };
            var bOptions = {
                url: '/user/<id>',
                params: {
                    id: '1',
                    id2: '2'
                }
            };
            var a = new _index.Fetcher(aOptions);
            var b = a.copy(bOptions);
            (0, _powerAssert2.default)(_rec14._expr(_rec14._capt(_rec14._capt(_rec14._capt(b, 'arguments/0/left/object').fullUrl, 'arguments/0/left') === '/api/user/1?id2=2', 'arguments/0'), {
                content: 'assert(b.fullUrl === \'/api/user/1?id2=2\')',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 166
            }));
        });

        it('should build fullUrl with custom serializeParams', function () {
            var _rec15 = new _powerAssertRecorder();

            var aOptions = {
                baseUrl: '/api',
                serializeParams: function serializeParams(url, params) {
                    return url + JSON.stringify(params);
                }
            };
            var bOptions = {
                url: '/user',
                params: {
                    id: '1',
                    id2: '2'
                }
            };
            var a = new _index.Fetcher(aOptions);
            var b = a.copy(bOptions);
            (0, _powerAssert2.default)(_rec15._expr(_rec15._capt(_rec15._capt(_rec15._capt(b, 'arguments/0/left/object').fullUrl, 'arguments/0/left') === _rec15._capt('/api/user' + _rec15._capt(_rec15._capt(JSON, 'arguments/0/right/right/callee/object').stringify(_rec15._capt(_rec15._capt(bOptions, 'arguments/0/right/right/arguments/0/object').params, 'arguments/0/right/right/arguments/0')), 'arguments/0/right/right'), 'arguments/0/right'), 'arguments/0'), {
                content: 'assert(b.fullUrl === \'/api/user\' + JSON.stringify(bOptions.params))',
                filepath: 'src/__tests__/FetcherTest.js',
                line: 184
            }));
        });
    });
});
//# sourceMappingURL=FetcherTest.js.map