'use strict';

var _powerAssertRecorder = function () { function PowerAssertRecorder() { this.captured = []; } PowerAssertRecorder.prototype._capt = function _capt(value, espath) { this.captured.push({ value: value, espath: espath }); return value; }; PowerAssertRecorder.prototype._expr = function _expr(value, source) { return { powerAssertContext: { value: value, events: this.captured }, source: source }; }; return PowerAssertRecorder; }();
/* eslint-env mocha */

var _powerAssert = require('power-assert');

var _powerAssert2 = _interopRequireDefault(_powerAssert);

var _sinon = require('sinon');

var _index = require('../index');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

describe('LoaderTest', function () {
    it('custom loader', function () {
        var result = 'test';
        var fakeFetch = (0, _sinon.spy)(function () {
            return Promise.resolve(result);
        });
        var options = {
            baseUrl: '/api',
            headers: {
                'Accept-Language': 'ru'
            },
            cacheable: true,
            method: 'GET',
            fetchFn: fakeFetch
        };

        var MyLoader = function (_Loader) {
            _inherits(MyLoader, _Loader);

            function MyLoader() {
                _classCallCheck(this, MyLoader);

                var opts = new _index.Fetcher(options);
                return _possibleConstructorReturn(this, _Loader.call(this, opts));
            }

            return MyLoader;
        }(_index.Loader);

        var loader = new MyLoader();
        return loader.fetch().then(function () {
            var _rec = new _powerAssertRecorder();

            (0, _powerAssert2.default)(_rec._expr(_rec._capt(_rec._capt(fakeFetch, 'arguments/0/object').calledOnce, 'arguments/0'), {
                content: 'assert(fakeFetch.calledOnce)',
                filepath: 'src/__tests__/LoaderTest.js',
                line: 36
            }));
        });
    });

    it('should call cacheable fetch once', function () {
        var result = 'test';
        var fakeFetch = (0, _sinon.spy)(function () {
            return Promise.resolve(result);
        });
        var options = {
            baseUrl: '/api',
            headers: {
                'Accept-Language': 'ru'
            },
            cacheable: true,
            method: 'GET',
            fetchFn: fakeFetch
        };
        var loader = new _index.Loader(new _index.Fetcher(options));
        return loader.fetch().then(function () {
            return loader.fetch();
        }).then(function () {
            var _rec2 = new _powerAssertRecorder();

            (0, _powerAssert2.default)(_rec2._expr(_rec2._capt(_rec2._capt(fakeFetch, 'arguments/0/object').calledOnce, 'arguments/0'), {
                content: 'assert(fakeFetch.calledOnce)',
                filepath: 'src/__tests__/LoaderTest.js',
                line: 56
            }));
        });
    });

    it('should call fetch after reset', function () {
        var result = 'test';
        var fakeFetch = (0, _sinon.spy)(function () {
            return Promise.resolve(result);
        });
        var options = {
            baseUrl: '/api',
            headers: {
                'Accept-Language': 'ru'
            },
            cacheable: false,
            method: 'GET',
            fetchFn: fakeFetch
        };
        var loader = new _index.Loader(new _index.Fetcher(options));
        return loader.fetch().then(function () {
            return loader.reset().fetch();
        }).then(function () {
            var _rec3 = new _powerAssertRecorder();

            (0, _powerAssert2.default)(_rec3._expr(_rec3._capt(_rec3._capt(fakeFetch, 'arguments/0/object').calledTwice, 'arguments/0'), {
                content: 'assert(fakeFetch.calledTwice)',
                filepath: 'src/__tests__/LoaderTest.js',
                line: 76
            }));
        });
    });
});
//# sourceMappingURL=LoaderTest.js.map