'use strict';

exports.__esModule = true;
exports.Repository = exports.Fetcher = exports.Loader = exports.HttpError = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };
/* eslint-env browser */


exports.mergeHeaders = mergeHeaders;
exports.checkStatus = checkStatus;
exports.createSerializeParams = createSerializeParams;

var _es6Error = require('es6-error');

var _es6Error2 = _interopRequireDefault(_es6Error);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Bound values to url placeholders or add query string
 */


/**
 * Input args for Fetcher
 *
 * @example
```js
{
    baseUrl: '/api',
    headers: {
        'Accept-Language': 'ru;q=0.8,en-US;q=0.6,en;q=0.4'
    },
    method: 'GET',
    postProcess,
    serializeParams
}
```
 */
function isFormData(val) {
    return typeof FormData !== 'undefined' && val instanceof FormData;
}

var toString = Object.prototype.toString;
function isBlob(val) {
    return toString.call(val) === '[object Blob]';
}

function isURLSearchParams(val) {
    return typeof URLSearchParams !== 'undefined' && val instanceof URLSearchParams;
}

/**
 * Merge headers objects.
 *
 * Uses global Headers class if exists.
 *
 * @example
 * ```js
 * // 
 * mergeHeaders({a: 1}, {a: 2, b: 3})
 * // If Headers not supported, result is object: {a: 2, b: 3}
 * // Else result is new Headers({a: 2, b: 3})
 * ```
 */
function mergeHeaders() {
    var isSupported = typeof Headers !== 'undefined';
    var result = isSupported ? new Headers() : {};

    for (var _len = arguments.length, headerSets = Array(_len), _key = 0; _key < _len; _key++) {
        headerSets[_key] = arguments[_key];
    }

    for (var i = 0, l = headerSets.length; i < l; i++) {
        var _headers = headerSets[i];
        if (!_headers || (typeof _headers === 'undefined' ? 'undefined' : _typeof(_headers)) !== 'object') {
            continue; // eslint-disable-line
        }

        if (isSupported && result instanceof Headers) {
            if (_headers instanceof Headers && Array.from) {
                var entries = Array.from(_headers.entries());
                for (var j = 0, k = entries.length; j < k; j++) {
                    var _entries$j = entries[j];
                    var name = _entries$j[0];
                    var v = _entries$j[1];

                    result.append(name, v);
                }
            } else {
                var _entries = Object.keys(_headers);
                for (var _j = 0, _k = _entries.length; _j < _k; _j++) {
                    result.append(_entries[_j], _headers[_entries[_j]]);
                }
            }
        } else {
            result = _extends({}, result, _headers);
        }
    }

    return result;
}

var HttpError = exports.HttpError = function (_Err) {
    _inherits(HttpError, _Err);

    function HttpError(response) {
        _classCallCheck(this, HttpError);

        var _this = _possibleConstructorReturn(this, _Err.call(this, response.statusText));

        _this.response = response;
        return _this;
    }

    return HttpError;
}(_es6Error2.default);

/**
 * Check response status value and throw error if it not in 200-300 range.
 */


function checkStatus(response) {
    if (response.status >= 200 && response.status < 300) {
        return response;
    }
    throw new HttpError(response);
}

function regExpMapString(replaceRegExp, template, params) {
    var newParams = _extends({}, params);
    var str = template.replace(replaceRegExp, function (v, k) {
        if (!params[k]) {
            throw new Error('No parameter provided to params: ' + k);
        }
        delete newParams[k];
        return params[k];
    });

    return {
        str: str,
        newParams: newParams
    };
}

var DEFAULT_MAP_REGEXP = new RegExp(':([\\w]+)', 'g');

/**
 * Create params serializer.
 */
function createSerializeParams(stringify) {
    var placeholderRegExp = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : DEFAULT_MAP_REGEXP;

    function serializeParams(url, params) {
        var _regExpMapString = regExpMapString(placeholderRegExp, url, params);

        var str = _regExpMapString.str;
        var newParams = _regExpMapString.newParams;

        var qStr = stringify(newParams);

        return str + (qStr ? '?' + qStr : '');
    }

    return serializeParams;
}

function compose(f1, f2) {
    return function (arg) {
        return f2(f1(arg));
    };
}

function pass(arg) {
    return arg;
}

/**
 * Cacheable data loader
 */
var Loader = exports.Loader = function () {
    function Loader(fetcher) {
        var _this2 = this;

        _classCallCheck(this, Loader);

        this._result = null;

        this._onError = function (err) {
            _this2.reset();
            throw err;
        };

        this._fetcher = fetcher || null;
    }

    Loader.prototype.fetch = function fetch() {
        var result = this._result;
        if (result) {
            return result;
        }

        result = this._fetch(this._fetcher).catch(this._onError);

        this._result = result;

        return result;
    };

    Loader.prototype._fetch = function _fetch(fetcher) {
        // eslint-disable-line
        if (!fetcher) {
            throw new Error('Fetcher is not initialized');
        }

        return fetcher.fetch();
    };

    Loader.prototype.reset = function reset() {
        this._result = null;
        return this;
    };

    return Loader;
}();

function defaultGetKey(rec) {
    var params = rec.params || {};
    return Object.keys(params).sort().map(function (key) {
        return key + ':' + params[key];
    }).join('.');
}

function callFetch(f) {
    return f.fetch();
}

/**
 * Fetch options builder
 */

var Fetcher = exports.Fetcher = function () {

    /**
     * Request options.
     */
    function Fetcher() {
        var rec = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

        _classCallCheck(this, Fetcher);

        this._baseUrl = rec.baseUrl || '/';
        this._serializeParams = rec.serializeParams;
        this._params = rec.params || null;
        this._url = rec.url || '';
        this.postProcess = rec.postProcess || pass;
        this._fetchFn = rec.fetchFn || (typeof fetch === 'undefined' ? pass : fetch);
        this.preProcess = rec.preProcess || null;
        var headers = rec.headers || {};

        var isPlainObject = false;
        var body = rec.body;
        if (body && (typeof body === 'undefined' ? 'undefined' : _typeof(body)) === 'object') {
            var isUrlSearchParams = isURLSearchParams(body);
            isPlainObject = !isFormData(body) && !isBlob(body) && !isUrlSearchParams;
            if (isUrlSearchParams || isPlainObject) {
                if (!headers) {
                    headers = {};
                }
                var ctxType = isUrlSearchParams ? 'application/x-www-form-urlencoded;charset=utf-8' : 'application/json';

                if (typeof Headers !== 'undefined' && headers instanceof Headers) {
                    if (!headers.has('Content-Type')) {
                        headers.set('Content-Type', ctxType);
                    }
                } else if (!headers['Content-Type']) {
                    headers['Content-Type'] = ctxType;
                }
            }
        }

        var options = {
            body: isPlainObject && body ? JSON.stringify(body) : body,
            headers: headers
        };
        if (rec.cache) options.cache = rec.cache;
        if (rec.credentials) options.credentials = rec.credentials;
        if (rec.integrity) options.integrity = rec.integrity;
        if (rec.method) options.method = rec.method;
        if (rec.mode) options.mode = rec.mode;
        if (rec.redirect) options.redirect = rec.redirect;
        if (rec.referrer) options.referrer = rec.referrer;
        if (rec.referrerPolicy) options.referrerPolicy = rec.referrerPolicy;

        this.options = options;

        if (this._params) {
            if (!this._serializeParams) {
                throw new TypeError('params exists, but no serializeParams method provided');
            }
            this.fullUrl = this._baseUrl + this._serializeParams(this._url, this._params);
        } else {
            this.fullUrl = this._baseUrl + this._url;
        }
    }

    /**
     * Create new copy of Fetcher with some options redefined.
     *
     * Headers will be merged with existing headers.
     * postProcessors will be composed with existing postProcessors.
     */


    /**
     * Generated full url from baseUrl, url and params.
     */


    Fetcher.prototype.copy = function copy(rec) {
        var headers = this.options.headers;
        return new this.constructor(_extends({
            baseUrl: rec.baseUrl || this._baseUrl,
            serializeParams: this._serializeParams,
            url: this._url
        }, this.options, rec, {
            postProcess: rec.postProcess ? compose(this.postProcess, rec.postProcess) : this.postProcess,
            params: this._params ? _extends({}, this._params, rec.params || {}) : rec.params,
            headers: rec.headers ? mergeHeaders(headers, rec.headers) : headers
        }));
    };

    Fetcher.prototype._fetch = function _fetch(f) {
        var result = this._fetchFn.call(null, f.fullUrl, f.options);
        return f.postProcess(result);
    };

    Fetcher.prototype.fetch = function fetch(rec) {
        var opts = rec ? this.copy(rec) : this;

        return this.preProcess ? this.preProcess(opts).then(callFetch) : this._fetch(opts);
    };

    return Fetcher;
}();

if (0) new (Function.prototype.bind.apply(Fetcher, [null].concat(0)))(); // eslint-disable-line

var Repository = exports.Repository = function () {
    function Repository(getKey) {
        _classCallCheck(this, Repository);

        this._clearAll = function (loader) {
            loader.reset();
        };

        this._loaders = new Map();
        this._getKey = getKey || defaultGetKey;
    }

    Repository.prototype.fetch = function fetch(params) {
        var key = this._getKey(params);
        var loader = this._loaders.get(key);
        if (!loader) {
            loader = new Loader(new Fetcher(params));
            this._loaders.set(key, loader);
        }

        return loader.fetch();
    };

    Repository.prototype.reset = function reset(params) {
        if (!params) {
            this._loaders.forEach(this._clearAll);
            return this;
        }

        var loader = this._loaders.get(this._getKey(params));
        if (loader) {
            loader.reset();
        }

        return this;
    };

    return Repository;
}();
//# sourceMappingURL=index.js.map